@@ -122,8 +122,6 @@ namespace dlib
         const std::string& title
     ) const
     {
-        using namespace std;
-
         if (!out)
             throw std::ios::failure("error occurred in cpp_pretty_printer_kernel_1::print");
 
@@ -172,7 +170,7 @@ namespace dlib
 
         int type;
         stack scopes; // a stack to hold old scopes
-        string token, temp;
+        std::string token, temp;
         t.get_token(type,token);
         while (type != tok::END_OF_FILE)
         {
@@ -188,7 +186,7 @@ namespace dlib
                     if (type == tok::WHITE_SPACE)
                     {
                         t.get_token(type,temp);
-                        if (temp.find_first_of("\n\r") != string::npos)
+                        if (temp.find_first_of("\n\r") != std::string::npos)
                             recently_seen_preprocessor = false;
                     }
                     if (t.peek_type() != tok::IDENTIFIER &&
@@ -322,7 +320,7 @@ namespace dlib
                     )
                     {
                         temp = token;
-                        istringstream sin(token);
+                        std::istringstream sin(token);
                         sin >> temp;
                         sin >> temp;
                         sin.get();
@@ -351,7 +349,7 @@ namespace dlib
             case tok::WHITE_SPACE: // -----------------------------------------
                 {
                     out << token;
-                    if (token.find_first_of("\n\r") != string::npos)
+                    if (token.find_first_of("\n\r") != std::string::npos)
                         recently_seen_preprocessor = false;
                 }
                 break;
@@ -497,10 +495,9 @@ namespace dlib
         const std::string& title
     ) const
     {
-        using namespace std;
-        ostringstream sout;
+        std::ostringstream sout;
         print(in,sout,title);
-        istringstream sin(sout.str());
+        std::istringstream sin(sout.str());
         number(sin,out);
     }
 
