@@ -271,6 +271,115 @@ namespace dlib
 
     };
 
+// ----------------------------------------------------------------------------------------
+
+    class input_rgb_image_pair
+    {
+        /*!
+            WHAT THIS OBJECT REPRESENTS
+                This input layer works with std::pair of RGB images of type matrix<rgb_pixel>.
+                It is useful when you want to input image pairs that are related to each other,
+                for instance, they are different distorted views of the same original image.
+                It is mainly supposed to be used with unsupervised loss functions such as
+                loss_barlow_twins_. You can also convert between input_rgb_image and
+                input_rgb_image_pair by copy construction or assignment.
+        !*/
+    public:
+        typedef std::pair<matrix<rgb_pixel>, matrix<rgb_pixel>> input_type;
+
+        input_rgb_image_pair (
+        );
+        /*!
+            ensures
+                - #get_avg_red()   == 122.782
+                - #get_avg_green() == 117.001
+                - #get_avg_blue()  == 104.298
+        !*/
+
+        input_rgb_image_pair (
+            float avg_red,
+            float avg_green,
+            float avg_blue
+        );
+        /*!
+            ensures
+                - #get_avg_red() == avg_red
+                - #get_avg_green() == avg_green
+                - #get_avg_blue() == avg_blue
+        !*/
+
+        inline input_rgb_image_pair (
+            const input_rgb_image& item
+        );
+        /*!
+            ensures
+                - #get_avg_red() == item.get_avg_red()
+                - #get_avg_green() == item.get_avg_green()
+                - #get_avg_blue() == item.get_avg_blue()
+        !*/
+
+        template <size_t NR, size_t NC>
+        inline input_rgb_image_pair (
+            const input_rgb_image_sized<NR, NC>& item
+        );
+        /*!
+            ensures
+                - #get_avg_red() == item.get_avg_red()
+                - #get_avg_green() == item.get_avg_green()
+                - #get_avg_blue() == item.get_avg_blue()
+        !*/
+
+        float get_avg_red(
+        ) const;
+        /*!
+            ensures
+                - returns the value subtracted from the red color channel.
+        !*/
+
+        float get_avg_green(
+        ) const;
+        /*!
+            ensures
+                - returns the value subtracted from the green color channel.
+        !*/
+
+        float get_avg_blue(
+        ) const;
+        /*!
+            ensures
+                - returns the value subtracted from the blue color channel.
+        !*/
+
+        void to_tensor (
+            forward_iterator ibegin,
+            forward_iterator iend,
+            resizable_tensor& data
+        ) const;
+        /*!
+            requires
+                - [ibegin, iend) is an iterator range over input_type objects.
+                - std::distance(ibegin,iend) > 0
+                - The input range should contain images that all have the same
+                  dimensions.
+            ensures
+                - Converts the iterator range into a tensor and stores it into #data.  In
+                  particular, if the input images have R rows, C columns then we will have:
+                    - #data.num_samples() == 2 * std::distance(ibegin,iend)
+                    - #data.nr() == R
+                    - #data.nc() == C
+                    - #data.k() == 3
+                  Moreover, each color channel is normalized by having its average value
+                  subtracted (according to get_avg_red(), get_avg_green(), or
+                  get_avg_blue()) and then is divided by 256.0.
+                  Additionally, the first elements in each pair are placed in the first half
+                  of the batch, and the second elements in the second half.
+        !*/
+
+        // Provided for compatibility with input_rgb_image_pyramid's interface
+        bool image_contained_point ( const tensor& data, const point& p) const { return get_rect(data).contains(p); }
+        drectangle tensor_space_to_image_space ( const tensor& /*data*/, drectangle r) const { return r; }
+        drectangle image_space_to_tensor_space ( const tensor& /*data*/, double /*scale*/, drectangle r ) const { return r; }
+
 // ----------------------------------------------------------------------------------------
 
     template <
@@ -610,6 +719,57 @@ namespace dlib
 
 // ----------------------------------------------------------------------------------------
 
+    class input_tensor
+    {
+        /*!
+            WHAT THIS OBJECT REPRESENTS
+                This input layer works with dlib::tensor objects. It is very similar to
+                the dlib::input layer except that it allows for concatenating data that
+                already resides in GPU memory.
+        !*/
+
+    public:
+        typedef tensor input_type;
+
+        input_tensor(
+        );
+        /*!
+            ensures
+                - input_tensor objects are default constructable
+        !*/
+
+        input_tensor(
+            const input_tensor& item
+        );
+        /*!
+            ensures
+                - input_tensor objects are copy constructable
+        !*/
+
+        template <typename forward_iterator>
+        void to_tensor(
+            forward_iterator ibegin,
+            forward_iterator iend,
+            resizable_tensor& data
+        ) const;
+        /*!
+            requires
+                - [ibegin, iend) is an iterator range over input_type objects.
+                - std::distance(ibegin,iend) > 0
+                - The input range should contain tensor objects that all have the same
+                  dimensions.
+            ensures
+                - Copies the iterator range into #data.  In particular, if the input tensors
+                  have R rows, C columns, and K channels then we will have:
+                    - #data.num_samples() == count_samples(ibegin,iend)
+                    - #data.nr() == R
+                    - #data.nc() == C
+                    - #data.k() == K
+                  This results in a tensor concatenation along the sample dimension.
+        !*/
+    };
+
+// ----------------------------------------------------------------------------------------
 }
 
 #endif // DLIB_DNn_INPUT_ABSTRACT_H_
