@@ -190,6 +190,57 @@ namespace dlib
             - fills the area defined by rect in the given image with the given pixel value.
     !*/
 
+// ----------------------------------------------------------------------------------------
+
+    template <typename image_type, typename pixel_type>
+    void draw_solid_convex_polygon (
+        image_type& image,
+        const polygon& poly,
+        const pixel_type& color,
+        const bool antialias = true,
+        const rectangle& area = rectangle(std::numeric_limits<long>::min(), std::numeric_limits<long>::min(),
+                                          std::numeric_limits<long>::max(), std::numeric_limits<long>::max())
+    );
+    /*!
+        requires
+            - image_type == an image object that implements the interface defined in
+              dlib/image_processing/generic_image.h
+            - pixel_traits<pixel_type> is defined
+        ensures
+            - Assigns all pixels in the given polygon with the given color.  Moreover, some regions
+              that are also in the convex hull of the polygon will also be filled as well.  In
+              particular, any row of pixels falling horizontally between two parts of the polygon
+              will be filled as well.
+            - When drawing the polygon, only the part of the polygon which overlaps both
+              the given image and area rectangle is drawn.
+            - Uses the given pixel color to draw the polygon.
+            - if (antialias == true) then we draw anti-aliased edges so they don't
+              look all pixely by alpha-blending them with the image.
+    !*/
+
+// ----------------------------------------------------------------------------------------
+
+    template <typename image_type, typename pixel_type>
+    void draw_solid_polygon (
+        image_type& image,
+        const polygon& poly,
+        const pixel_type& color,
+        const rectangle& area = rectangle(std::numeric_limits<long>::min(), std::numeric_limits<long>::min(),
+                                          std::numeric_limits<long>::max(), std::numeric_limits<long>::max())
+    );
+    /*!
+        requires
+            - image_type == an image object that implements the interface defined in
+              dlib/image_processing/generic_image.h
+            - pixel_traits<pixel_type> is defined
+        ensures
+            - Draws the given polygon onto the image, filling in the interior and edges of the
+              polygon.
+            - When drawing the polygon, only the part of the polygon which overlaps both
+              the given image and area rectangle is drawn.
+            - Uses the given pixel color to draw the polygon.
+    !*/
+
 // ----------------------------------------------------------------------------------------
 
     template <
